#!/usr/bin/env python
# coding=utf-8

import os
import sys
import json
import logging
import six

if six.PY2:
    import cPickle as pickle
else:
    import pickle

try:
    import redis
except ImportError:
    pass


class RedisCache(object):
    def __init__(self, host='localhost', port=6379, db=0, decode_responses=False):
        self.host = host
        self.port = port
        self.db = db
        try:
            _cache = redis.StrictRedis(host=self.host, port=self.port, db=self.db, decode_responses=decode_responses)
        except Exception as e:
            logging.warning(e)
            logging.warning('Connect redis failed. The cache will not work')
            return None
        self._cache = _cache
        self._key_sep = '\x01'
        self._expire = None

    def set_key_sep(self, sep):
        self._key_sep = sep

    def set_expire(self, time):
        self._expire = time

    def _wrap_key(
            self,
            k,
    ):
        if isinstance(k, (list, tuple)):
            return self._key_sep.join(k)
        else:
            return k

    def get(self, k):
        k = self._wrap_key(k)
        return self._cache.get(k)

    def set(self, k, v, ex=None):
        k = self._wrap_key(k)
        if ex is None:
            ex = self._expire
        return self._cache.set(k, v, ex=ex)

    def get_list(self, k, sep='\x01'):
        k = self._wrap_key(k)
        v = self._cache.get(k)
        if v is None:
            return None
        return v.split(sep)

    def set_list(self, k, v, sep='\x01', ex=None):
        k = self._wrap_key(k)
        if v is None:
            nv = None
        else:
            nv = sep.join(v)
        if ex is None:
            ex = self._expire
        return self._cache.set(k, nv, ex=ex)

    def get_json(self, k):
        k = self._wrap_key(k)
        jsn_str = self._cache.get(k)
        if jsn_str is None:
            return None
        else:
            return json.loads(jsn_str)

    def set_json(self, k, v, ex=None):
        k = self._wrap_key(k)
        if v is None:
            nv = None
        else:
            nv = json.dumps(v, ensure_ascii=False)
        if ex is None:
            ex = self._expire
        return self._cache.set(k, nv, ex=ex)

    def get_obj(self, k):
        k = self._wrap_key(k)
        s = self._cache.get(k)
        if s is None:
            return None
        else:
            return pickle.loads(s)

    def set_obj(self, k, v, ex=None):
        k = self._wrap_key(k)
        if v is None:
            nv = None
        else:
            nv = pickle.dumps(v, protocol=2)
        if ex is None:
            ex = self._expire
        return self._cache.set(k, nv, ex=ex)

    def delete(self, *ks):
        """
        ks: one or list of keys
        """
        return self._cache.delete(*ks)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import string
import hashlib

CHINESE_PUNCTUATION = u'，。！￥？——；“”：《》（）'
ENGLISH_PUNCTUATION = string.punctuation


def is_chinese_punctuation(char):
    return char in CHINESE_PUNCTUATION


def remove_chinese_punctuation(s):
    ret = u''
    for c in s:
        if not is_chinese_punctuation(c):
            ret += c
    return ret


def is_english_punctuation(char):
    return char in ENGLISH_PUNCTUATION


def remove_english_punctuation(s):
    ret = u''
    for c in s:
        if not is_english_punctuation(c):
            ret += c
    return ret


def is_pure_english(s):
    yes = True
    for e in s:
        if ord(e) > 127:
            yes = False
            break
    if yes:
        return True
    else:
        return False


def english_words(s):
    return ''.join([e for e in s if ord(e) <= 127]).strip()


def common_suffix(s1, s2):
    """
    get common suffix of two string
    @param s1: first string
    @param s2: second string
    @return: the common suffix

    """
    min_len = min(len(s1), len(s2))
    if min_len == 0:
        return ''
    for i in range(-1, -min_len - 1, -1):
        if s1[i] != s2[i]:
            break
    if i == -1:
        return ''
    elif i == -min_len:
        return s1[i:]
    else:
        return s1[i + 1:]


def md5(s):
    return hashlib.md5(s.encode('utf-8')).hexdigest()


def test():
    def print_list(lst):
        for e in lst:
            print e.encode('utf-8')


if __name__ == '__main__':
    test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
import sys
import json


class Logger(object):
    def __init__(self,
            log_path,
            level=logging.INFO,
            stdout=False,
            when="D",
            backup=7,
            format="%(levelname)s: %(asctime)s: %(filename)s:%(lineno)d * %(thread)d %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S"):
        """
        init_log - initialize log module

        Args:
          log_path      - Log file path prefix.
                          Log data will go to two files: log_path.log and log_path.log.wf
                          Any non-exist parent directories will be created automatically
          level         - msg above the level will be displayed
                          DEBUG < INFO < WARNING < ERROR < CRITICAL
                          the default value is logging.INFO
          when          - how to split the log file by time interval
                          'S' : Seconds
                          'M' : Minutes
                          'H' : Hours
                          'D' : Days
                          'W' : Week day
                          default value: 'D'
          format        - format of the log
                          default format:
                          %(levelname)s: %(asctime)s: %(filename)s:%(lineno)d * %(thread)d %(message)s
                          INFO: 12-09 18:02:42: log.py:40 * 139814749787872 HELLO WORLD
          backup        - how many backup file to keep
                          default value: 7

        Raises:
            OSError: fail to create log directories
            IOError: fail to open log file

        Example:
        init_log("./log/my_program")  # 日志保存到./log/my_program.log和./log/my_program.log.wf，按天切割，保留7天
        logging.info("Hello World!!!")

        """
        self._logger = None

        formatter = logging.Formatter(format, datefmt)
        logger = logging.getLogger()
        logger.setLevel(level)

        dir = os.path.dirname(log_path)
        if not dir:
            dir = './'
        if not os.path.isdir(dir):
            os.makedirs(dir)

        if stdout:
            stdout_handler = logging.StreamHandler(sys.stdout)
            stdout_handler.setLevel(level)
            stdout_handler.setFormatter(formatter)
            logger.addHandler(stdout_handler)

        handler = logging.handlers.TimedRotatingFileHandler(
            log_path + ".log", when=when, backupCount=backup)
        handler.setLevel(level)
        handler.setFormatter(formatter)
        # https://www.jianshu.com/p/25f70905ae9d
        handler.suffix = '%Y-%m-%d'
        logger.addHandler(handler)

        err_handler = logging.handlers.TimedRotatingFileHandler(
            log_path + ".log.wf", when=when, backupCount=backup)
        err_handler.setLevel(logging.WARNING)
        err_handler.setFormatter(formatter)
        handler.suffix = '%Y-%m-%d'
        logger.addHandler(err_handler)
        self._logger = logger
        return logger

    def convert_msg(self, msg):
        if isinstance(msg, dict):
            _msg = json.dumps(msg, ensure_ascii=False)
        else:
            _msg = msg
        return _msg

    def debug(self, msg, *args, **kwargs):
        _msg = self.convert_msg(msg)
        self._logger.debug(_msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        _msg = self.convert_msg(msg)
        self._logger.info(_msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        _msg = self.convert_msg(msg)
        self._logger.warning(_msg, *args, **kwargs)

    warn = warning

    def error(self, msg, *args, **kwargs):
        _msg = self.convert_msg(msg)
        self._logger.error(_msg, *args, **kwargs)


    def exception(self, msg, *args, **kwargs):
        _msg = self.convert_msg(msg)
        self._logger.exception(_msg, *args, **kwargs)


    def critical(self, msg, *args, **kwargs):
        _msg = self.convert_msg(msg)
        self._logger.critical(_msg, *args, **kwargs)

    def ls_msg(self, es_index, es_type, msg):
        if isinstance(msg, dict):
            _msg = json.dumps(msg, ensure_ascii=False)
        else:
            _msg = msg
        _msg = u'logstash###{}###{}###{}'.format(es_index, es_type, _msg)
        return _msg

    def ls_debug(self, es_index, es_type, msg):
        """
        logstash debug
        """
        _msg = self.ls_msg(es_index, es_type, msg)
        self._logger.debug(_msg)

    def ls_info(self, es_index, es_type, msg):
        _msg = self.ls_msg(es_index, es_type, msg)
        self._logger.info(_msg)

    def ls_warning(self, es_index, es_type, msg):
        _msg = self.ls_msg(es_index, es_type, msg)
        self._logger.warning(_msg)

    ls_warn = ls_warning

    def ls_error(self, es_index, es_type, msg):
        _msg = self.ls_msg(es_index, es_type, msg)
        self._logger.error(_msg)

    def ls_critical(self, es_index, es_type, msg):
        _msg = self.ls_msg(es_index, es_type, msg)
        self._logger.critical(_msg)
